// config: norepeat

// Distribute_pilou.js v.0.1 - Marco (mkdm), 2017
// based on :
// 		Distribute.js original script by Michael Gibson

#include "GetObjects.js"

function WrapWithObjectList( obj )
{
	var list = moi.geometryDatabase.createObjectList();
	list.addObject( obj );
	return list;
}

function Cancel( factories )
{
	for ( var i = 0; i < factories.length; ++i )
		factories[i].cancel();
		
	factories.length = 0;
}

function Commit( factories )
{
	for ( var i = 0; i < factories.length; ++i )
		factories[i].commit();
}

function Update( objects, factories, manual_mode, distance )
{
	Cancel( factories );
	
	// dir is a string value one of 'x', 'y', 'z'
	var dir = moi.ui.commandUI.dir.value;
	
	// Sort the objects by their bounding box center points.

	var sorted = [];
	
	var target_space_between;
	var idxOffset;
	
	for ( var i = 0; i < objects.length; ++i )
	{
		var obj = objects.item(i);
		var bbox = obj.getBoundingBox( true ); // true = Use high accuracy bounds
		var entry = { object:obj, bounds:bbox, sortval:bbox.center[dir] };
		sorted.push( entry );
	}

	sorted.sort( function(a,b){ return a.sortval - b.sortval } );
	
	if (manual_mode) {
		target_space_between = distance;
		idxOffset = 0;
	} else {
		var total_space = sorted[sorted.length-1].bounds.min[dir] - sorted[0].bounds.max[dir];
			
		var occupied_space = 0;
		for ( var i = 1; i < sorted.length-1; ++i )
			occupied_space += sorted[i].bounds.max[dir] - sorted[i].bounds.min[dir];
			
		var available_space = total_space - occupied_space;
			
		target_space_between = available_space / (sorted.length-1);
		
		idxOffset = 1;
	}
	
	var prev_end = sorted[0].bounds.max[dir];
	
	for ( var i = 1; i < sorted.length - idxOffset; ++i )
	{
		var current_dist_between = sorted[i].bounds.min[dir] - prev_end;
		var delta = target_space_between - current_dist_between;
		
		var pt = moi.vectorMath.createPoint(0,0,0);
		pt[dir] = delta;
		
		var factory = moi.command.createFactory( 'move' );
		factory.setInput( 0, WrapWithObjectList(sorted[i].object) );
		factory.setInput( 1, moi.vectorMath.createPoint(0,0,0) );
		factory.setInput( 2, pt );
		factory.update();
		factories.push( factory );
		
		prev_end = sorted[i].bounds.max[dir] + delta;
	}
}

function DoDistribute()
{
	var objectpicker = moi.ui.createObjectPicker();
	if ( !GetObjects( objectpicker ) )
		return;
		
	var objects = objectpicker.objects;
	if ( objects.length < 3 )
	{
		moi.ui.alert( 'Erreur - Ncessite la Slection de 3 Objets ou plus.' );
		return;
	}
	
	moi.ui.beginUIUpdate();
	moi.ui.hideUI( 'SelectPrompt' );
	moi.ui.showUI( 'OptionsPrompt' );
	moi.ui.showUI( 'DistributeOptions' );
	moi.ui.endUIUpdate();
	
	var factories = [];
	var manual_mode = moi.ui.commandUI.manual_mode.value;
	var distance = moi.ui.commandUI.distance.numericValue;

	Update( objects, factories, manual_mode, distance );	
	
	var dlg = moi.ui.commandDialog;
	while ( 1 )
	{
		if ( !dlg.waitForEvent() )
			return;
			
		if ( dlg.event == 'done' )
			break;
					
		if ((dlg.event == 'dir') || (dlg.event == 'manual_mode') || (dlg.event == 'distance')) {
			manual_mode = moi.ui.commandUI.manual_mode.value;
			distance = moi.ui.commandUI.distance.numericValue;
			Update(objects, factories, manual_mode, distance);
		}
	}
	
	Commit( factories );
}

DoDistribute();
