//	Enneper minimal surface by Brian McMillin February 16, 2014
//  The nearly identical parametric equations come from https://sites.google.com/site/workofap/surface
//	or http://rhino3.de/_develop/__v3_plugins/math/library.shtml 
//	r = u, is the radius parameter whose interval is (0 to 1). (Includes 0 and 1). (Or slightly more than 1 can be used)
//	phi = v, is the rotational angle parameter whose interval is (-pi, pi]. In other words, -pi <= phi < pi, so pi is excluded from the interval.
//	lobenum is the number of pairs of lobes, or trigonometry cycles, or symmetry number.

function Cancel( factories )
{
	for ( var i = 0; i < factories.length; ++i )
		factories[i].cancel();
		
	factories.length = 0;
}

function Commit( factories )
{
	for ( var i = 0; i < factories.length; ++i )
		factories[i].commit();
}

function EvalEnneper( tu, tv, lobenum )
{
	var phi = Math.PI * (2.0 * tv - 1.0);
	var r = tu;
	var m = 2 * lobenum -1;
	// Define x, y, and z values.
	var x =	r * Math.cos(phi) - Math.pow(r,m) * Math.cos(m*phi) / m;
	var y =	r * Math.sin(phi) + Math.pow(r,m) * Math.sin(m*phi) / m;	
	var z = 2 * Math.pow(r,lobenum) * Math.cos(lobenum*phi) / lobenum; 
 	
	return moi.vectorMath.createPoint( x, y, z );
}

function Update( minr, maxr, lobenum, unum, vnum, ptsnum, factories )
{
	Cancel( factories );
//	var lobes = lobenum;
	if (lobenum <= 0)
	{
		lobenum = 2;
	}
	
//	Plot the "kind of concentric" curves.	
	for (var j = 0; j < (unum); ++j )	
	{
	var factory = moi.command.createFactory( 'interpcurve' );
//	var factory = moi.command.createFactory( 'curve' );
	var tu = j / (unum - 1);
	tu = tu * (maxr - minr) + minr;
	
		for ( var i = 0; i < (ptsnum); ++i )
		{
			var tv = i / (ptsnum - 1);
		
			var pt = EvalEnneper( tu, tv, lobenum );		
		
			factory.createInput( 'point' );
			factory.setInput( factory.numInputs - 1, pt );
		}

		factories.push( factory );
//	The next line is a test addition, display the current factory, (one curve).  It works well...
		factory.update();
	}

	// Build "kind of radial" curves if box checked.		
	if ( moi.ui.commandUI.RadialLinesCheckbox.value )
	{	
		for (var j = 0; j < (vnum - 1); ++j )
		{
			var factory = moi.command.createFactory( 'interpcurve' );
			var tv = j / (vnum - 1);

			for ( var i = 0; i < ptsnum; ++i )
			{
				var tu = i / (ptsnum - 1);
				tu = tu * (maxr - minr) + minr;
			
				var pt = EvalEnneper( tu, tv, lobenum );		
		
				factory.createInput( 'point' );
				factory.setInput( factory.numInputs - 1, pt );
			}

			factories.push( factory );
			//	Display the current factory.
			factory.update();
		}	
	}
		for ( var i = 0; i < factories.length; ++i )
		{
			factories[i].update();
		}	
}

function DoEnneper()
{
	var factories = new Array();
	var ui = moi.ui.commandUI
	
	Update( ui.minr.numericValue, ui.maxr.numericValue, ui.lobenum.integerValue, ui.unum.integerValue, ui.vnum.integerValue, moi.ui.commandUI.ptsnum.integerValue, factories );

	var dialog = moi.ui.commandDialog;

	while ( 1 )
	{
		if ( !dialog.waitForEvent() )
		{
			Cancel( factories );
			return;
		}
			
		if ( dialog.event == 'done' )
			break;
			
		if ( dialog.event == 'minr' || dialog.event == 'maxr' || dialog.event == 'lobenum' || dialog.event == 'unum' || dialog.event == 'vnum' || dialog.event == 'ptsnum' || dialog.event == 'RadialLinesCheckbox' )
			Update( ui.minr.numericValue, ui.maxr.numericValue, ui.lobenum.integerValue, ui.unum.integerValue, ui.vnum.integerValue, moi.ui.commandUI.ptsnum.integerValue, factories );
	}
			Commit( factories );
}

DoEnneper();
