/*
Garlic is a script to model a garlic bulb made up of cloves of garlic.
http://annals-wuls.sggw.pl/files/files/agriculture/afe2012no59full.pdf#page=77 

Rebuild and loft may be used. 

Garlic cloves may be easily skinned by vigorously shaking them in two stainless steel bowls for 20 seconds.
https://www.youtube.com/watch?v=Dc7w_PGSt9Y

Recipe for a fluffy garlic paste, which lasts a month in the refrigerator:
http://allrecipes.com/recipe/thick-style-lebanese-garlic-sauce/?mxt=t06dda
http://www.mamaslebanesekitchen.com/dips/lebanese-garlic-dip/#sthash.IMG93lue.dpbs
On the radio, using light canola oil instead, was recommended.

Moi javascript version by Brian McMillin, August 12, 2015.

*/

function Cancel( factories )
{
	for ( var i = 0; i < factories.length; ++i )
		factories[i].cancel();
		
	factories.length = 0;
}

function Commit( factories )
{
	for ( var i = 0; i < factories.length; ++i )
		factories[i].commit();
}

function EvalG( tu, t2, d1, d2, height, cloves, rShape, lShape, aShape, bShape, cShape )
{
	var phi_j;
	var phi_i;
	var psi_j;
	var psi_i;	
	var n = 2;  // n is for two halves of a closed curve.
	phi_j =  Math.PI * 2.0 * tu;
	psi_j = Math.PI * tu / 5.0;
	phi_i =  Math.PI * 2.0 * t2;
	psi_i = Math.PI * t2 / 5.0;
	
	// Define x, y, and z values.
	var x = (phi_j/n) * Math.sin(phi_j/n) * ((cloves + rShape) * Math.cos(phi_i) - lShape * rShape * Math.cos(((cloves + rShape)/rShape) * phi_i)) ;	
	
	var y = (phi_j/n) * Math.sin(phi_j/n) * ((cloves + rShape) * Math.sin(phi_i) - lShape * rShape * Math.sin(((cloves + rShape)/rShape) * phi_i));	
	
	var z = height * Math.cos((aShape * psi_j) + bShape) * Math.sin((psi_j/n) + cShape); 	

	return moi.vectorMath.createPoint( x, y, z );
	// Note psi_j is used for z, NOT psi_i.
	// Longitude lines could be coded for, but are not needed for loft.
}

function Update( d1, d2, height, cloves, rShape, lShape, aShape, bShape, cShape, uDensity, vDensity, numpoints, factories )
{
	Cancel( factories );
	
//  Plot "latitude curves."	
	for (var i = 0; i < (uDensity-1); ++i )
	{
	var factory = moi.command.createFactory( 'interpcurve' );
//	var factory = moi.command.createFactory( 'curve' );
	var tu = i / (uDensity - 1);

	for ( var j = 0; j < (numpoints); ++j )
		{
			var t2 = j / (numpoints - 1);
			var pt = EvalG( tu, t2, d1, d2, height, cloves, rShape, lShape, aShape, bShape, cShape );		
			factory.createInput( 'point' );
			factory.setInput( factory.numInputs - 1, pt );
		}
		factories.push( factory );
		factory.update();
	}

//	Plot "longitude curves."	
	for (var i = 0; i < (vDensity-1); ++i )
	{
	var factory = moi.command.createFactory( 'interpcurve' );
//	var factory = moi.command.createFactory( 'curve' );
	var tv = i / (vDensity - 1);

		for ( var j = 0; j < (numpoints); ++j )
		{
			var t2 = j / (numpoints - 1);
			var pt = EvalG( t2, tv, d1, d2, height, cloves, rShape, lShape, aShape, bShape, cShape );		
			factory.createInput( 'point' );
			factory.setInput( factory.numInputs - 1, pt );
		}

		factories.push( factory );
		factory.update();
	}
	
	
		for ( var i = 0; i < factories.length; ++i )
		{
			factories[i].update();
		}	
}

function DoGarlic()
{
	var factories = new Array();
	var UI = moi.ui.commandUI
	
	Update( UI.d1.numericValue, UI.d2.numericValue, UI.height.numericValue, UI.cloves.integerValue, UI.rShape.numericValue, UI.lShape.numericValue, UI.aShape.numericValue, UI.bShape.numericValue, UI.cShape.numericValue, UI.uDensity.integerValue, UI.vDensity.integerValue, UI.numpoints.integerValue, factories );

	var dialog = moi.ui.commandDialog;

	while ( 1 )
	{
		if ( !dialog.waitForEvent() )
		{
			Cancel( factories );
			return;
		}
			
		if ( dialog.event == 'done' )
			break;
			
		if ( dialog.event == 'd1' || dialog.event == 'd2' || dialog.event == 'height' || dialog.event == 'cloves' || dialog.event == 'rShape' || dialog.event == 'lShape' || dialog.event == 'aShape' || dialog.event == 'bShape' || dialog.event == 'cShape' || dialog.event == 'uDensity' || dialog.event == 'vDensity' || dialog.event == 'numpoints' )
			Update( UI.d1.numericValue, UI.d2.numericValue, UI.height.numericValue, UI.cloves.integerValue, UI.rShape.numericValue, UI.lShape.numericValue, UI.aShape.numericValue, UI.bShape.numericValue, UI.cShape.numericValue, UI.uDensity.integerValue, UI.vDensity.integerValue, UI.numpoints.integerValue, factories );
	}
			Commit( factories );
}

DoGarlic();
