// Hatch selected closed curves
// Version from March 1, 2015

// NOTE
// Works in TOP view only
// If you want to hatch several closed curves at once they must be all on the same Z plane.

// HINT
// You can hide all hatch lines In the Scene browser under Objects.
// Or you can select and delete or color all hatch lines.


#include "WaitForDialogDone.js"

function hatchit()
{
	var hatchlineOffset = moi.ui.commandUI['offset'].numericValue * 1.0;
	
	var objects = moi.geometryDatabase.getSelectedObjects();
	var bbox = objects.getHighAccuracyBoundingBox();
	var height = bbox.max.y - bbox.min.y
	
	//Draw first hatch line
	var factory = moi.command.createFactory( "line" );
	factory.setInput( 0, moi.vectorMath.createPoint( bbox.min.x, bbox.max.y, bbox.min.z ) );
	factory.setInput( 1, moi.vectorMath.createPoint( bbox.min.x - height, bbox.min.y, bbox.min.z ) );
	factory.update();
	var hatchline = factory.getCreatedObjects();
	
	//Hatch over object
	var factory = moi.command.createFactory( "arraydir" );
	factory.setInput( 0, hatchline );
	factory.setInput( 2, moi.vectorMath.createPoint( bbox.min.x - height, bbox.min.y, bbox.min.z ) ); //BasePt
	
	factory.setInput( 4, moi.vectorMath.createPoint( bbox.max.x, bbox.min.y, bbox.min.z ) ); //ExtentPt
	factory.setInput( 5, 'Offset, Extent' ); //Mode
	
	factory.disableUpdate( false );
	
	var dlg = moi.ui.commandDialog;
	
	while ( 1 )
	{
		
	try {
		if ( moi.ui.commandUI.offset.value != 0 )
		hatchlineOffset = moi.ui.commandUI['offset'].numericValue * 1.0;
		factory.setInput( 3, moi.vectorMath.createPoint( bbox.min.x - height + hatchlineOffset, bbox.min.y, bbox.min.z ) ); //OffsetPt
		factory.update();
	}
	catch(e) {
	}
	
	
		if ( !dlg.waitForEvent() )
			return false; // Canceled
			
		if ( dlg.event == 'done' )
			break;
	}
	
	var hatchlines = factory.getCreatedObjects();
	hatchlines.setProperty( 'name', "Hatch lines" );	
	factory.commit();
	
	//Trim hatch lines
	var factory = moi.command.createFactory( 'booleanintersection' );
	factory.setInput( 0, hatchlines );
	factory.setInput( 1, objects );
	factory.commit();
	
	var crosshatch = moi.ui.commandUI['crosshatch'].value;
	if ( crosshatch)
	{
		//Draw first cross hatch line
		var factory = moi.command.createFactory( "line" );
		factory.setInput( 0, moi.vectorMath.createPoint( bbox.min.x - height, bbox.max.y, bbox.min.z) );
		factory.setInput( 1, moi.vectorMath.createPoint( bbox.min.x, bbox.min.y, bbox.min.z ) );
		factory.update();
		var crosshatchline = factory.getCreatedObjects();
		
		//Cross hatch over object
		var factory = moi.command.createFactory( "arraydir" );
		factory.setInput( 0, crosshatchline );
		factory.setInput( 2, moi.vectorMath.createPoint( bbox.min.x, bbox.min.y, bbox.min.z ) ); //BasePt
		factory.setInput( 3, moi.vectorMath.createPoint( bbox.min.x + hatchlineOffset, bbox.min.y, bbox.min.z ) ); //OffsetPt
		factory.setInput( 4, moi.vectorMath.createPoint( bbox.max.x + height, bbox.min.y, bbox.min.z ) ); //ExtentPt
		factory.setInput( 5, 'Offset, Extent' ); //Mode
		factory.update();
		var crosshatchlines = factory.getCreatedObjects();
		crosshatchlines.setProperty( 'name', "Hatch lines" );
		factory.commit();
		
		//Trim cross hatch lines
		var factory = moi.command.createFactory( 'booleanintersection' );
		factory.setInput( 0, crosshatchlines );
		factory.setInput( 1, objects );
		factory.commit();
	}

}

hatchit();

