//	Henneberg minimal surface by Brian McMillin February 20, 2014
//   
//	phi = u, is an angle parameter whose interval is [-1, 1]. Excludes -1 and 1). 
//	tau = v, is an angle parameter whose interval is [-PI/2, PI/2]. Excludes -PI/2 and PI/2

function Cancel( factories )
{
	for ( var i = 0; i < factories.length; ++i )
		factories[i].cancel();
		
	factories.length = 0;
}

function Commit( factories )
{
	for ( var i = 0; i < factories.length; ++i )
		factories[i].commit();
}

function EvalHenneberg( tu, tv )
{
	var phi = 2*tu - 1.0;
//	var tau = Math.PI * (2.0 * tv - 1.0)/2;
	var tau = Math.PI * (tv - 0.5);	
	sinhphi = Math.exp(phi) - Math.exp(-phi); //Divide by 2 omitted, as 2/2 =1.
	sinh3phi = Math.exp(3*phi) - Math.exp(-3*phi);
	cosh2phi = Math.exp(2*phi) + Math.exp(-2*phi);
	// Define x, y, and z values.
	var x = sinhphi*Math.cos(tau) - sinh3phi*Math.cos(3*tau)/3;
	var y = sinhphi*Math.sin(tau) + sinh3phi*Math.sin(3*tau)/3;
	var z = cosh2phi*Math.cos(2*tau);
 
	return moi.vectorMath.createPoint( x, y, z );
}

//function Update( unum, vnum, ptsnum, factories )
function Update( unum, vnum, ptsnum, factories )
{
	Cancel( factories );
	
	//	Plot the "kind of concentric" curves.
	if ( moi.ui.commandUI.ConcentricCurvesCheckbox.value )	
	{
		for (var j = 0; j < (unum); ++j )		
		{
			var factory = moi.command.createFactory( 'interpcurve' );
			var tu = j / (unum-1);
			// -1 and 1 must be excluded. 0 becomes -1;			
			if (tu == 0)
				tu = 0.00001;
			if (tu == 1)
				tu = 0.99999;
			for ( var i = 0; i < (ptsnum); ++i )		
			{
				var tv = i / (ptsnum - 1);
			// -PI/2 and PI/2 must be excluded. 0 becomes -PI/2. 1 becomes PI/2.  				
				if (tv == 0)
					tv = 0.00001;
				if (tv == 1)
					tv = 0.99999;					
				var pt = EvalHenneberg( tu, tv );		
		
				factory.createInput( 'point' );
				factory.setInput( factory.numInputs - 1, pt );
			}

		factories.push( factory );
		//  Display the current factory.
		factory.update();
	}
	}
	
	// Build "kind of radial" curves if box checked.
	if ( moi.ui.commandUI.RadialCurvesCheckbox.value )
	{	
		for (var j = 0; j < (vnum); ++j )		
		{
			var factory = moi.command.createFactory( 'interpcurve' );
			var tv = j / (vnum - 1);
			// -PI/2 and PI/2 must be excluded. 0 becomes -PI/2. 1 becomes PI/2.  				
				if (tv == 0)
					tv = 0.00001;
				if (tv == 1)
					tv = 0.99999;				
			for ( var i = 0; i < ptsnum; ++i )
			{
				var tu = i / (ptsnum - 1);
				// -1 and 1 must be excluded. 0 becomes -1;			
				if (tu == 0)
					tu = 0.00001;
				if (tu == 1)
					tu = 0.99999;
				var pt = EvalHenneberg( tu, tv );		
		
				factory.createInput( 'point' );
				factory.setInput( factory.numInputs - 1, pt );
			}

			factories.push( factory );
			//	Display the current factory.
			factory.update();
		}	
	}
		for ( var i = 0; i < factories.length; ++i )
		{
			factories[i].update();
		}	
}

function DoHenneberg()
{
	var factories = new Array();
	var ui = moi.ui.commandUI
	
	Update( ui.unum.integerValue, ui.vnum.integerValue, moi.ui.commandUI.ptsnum.integerValue, factories );

	var dialog = moi.ui.commandDialog;

	while ( 1 )
	{
		if ( !dialog.waitForEvent() )
		{
			Cancel( factories );
			return;
		}
			
		if ( dialog.event == 'done' )
			break;
			
		if ( dialog.event == 'unum' || dialog.event == 'vnum' || dialog.event == 'ptsnum' || dialog.event == 'RadialCurvesCheckbox' || dialog.event == 'ConcentricCurvesCheckbox' )
			Update( ui.unum.integerValue, ui.vnum.integerValue, moi.ui.commandUI.ptsnum.integerValue, factories );
	}
			Commit( factories );
}

DoHenneberg();
