// Draw Koch Snowflake using recursion.
// Brian McMillin January 17, 2014

/*
The level 1 Koch snowflake is a triangle, with Top point = (topX, topY),
Left point = (leftX, leftY), and Right point = (rightX, rightY).
The snowflake consists of three Koch curves, one for each side of the initial triangle.
So the recursion is called 3 times, once for the left leg, once for the right leg,
and once for the base of the triangle.

References:
1.  The book: JAVA software solutions, John Lewis and William Loftus.  (Their code has errors.)
2.  http://aisha91.hubpages.com/hub/Java-Source-Code-Recursive-Snow-Flakes

Two points, which represent the endpoints of one side of the triangle, are used by
the recursive function KochSnow, as well as the level of subdivision.
The KochSnow function is called two more times, first using side 2 of the triangle along
with its endpoints, and similarly for side 3 of the triangle.

So either the 3 points are entered manually, or else the script could be set up for the
user to manually select triangle(s).

*/

function alert( msg )
{
	moi.ui.commandUI.alert( msg );
}

function Cancel( factories )
{
	for ( var i = 0; i < factories.length; ++i )
		factories[i].cancel();
		
	factories.length = 0;
}

function Commit( factories )
{
	for ( var i = 0; i < factories.length; ++i )
		factories[i].commit();
}

// The Koch function is recursive.
function KochSnow(x1, y1, x5, y5, lev, factories)
{
	var deltaX, deltaY, x2, y2, x3, y3, x4, y4;
	var z = 0.0; // 2D curve.
	
	if (lev == 0)
	{
		var linefactory = moi.command.createFactory( 'line' );
		var pt1 =  moi.vectorMath.createPoint( x1, y1, z );
		var pt2 =  moi.vectorMath.createPoint( x5, y5, z );
		linefactory.setInput( 0, pt1 );
		linefactory.setInput( 1, pt2 );
		factories.push( linefactory );
	}
	else
	{
		deltaX = x5 - x1; // distance between end points
		deltaY = y5 - y1;
		
		x2 = x1 + deltaX/3; // one third
		y2 = y1 + deltaY/3;
		
		x3 = (x1 + x5 )/2 + Math.sqrt(3) * (y1 - y5 )/6; 
		y3 = (y1 + y5 )/2 + Math.sqrt(3) * (x5 - x1 )/6;
		
		x4 = x1 + deltaX*2/3; // two thirds
		y4 = y1 + deltaY*2/3;
		
		KochSnow(x1, y1, x2, y2, lev-1, factories);
		KochSnow(x2, y2, x3, y3, lev-1, factories);
		KochSnow(x3, y3, x4, y4, lev-1, factories);
		KochSnow(x4, y4, x5, y5, lev-1, factories);
	}

}

function Update(topX, topY, leftX, leftY, rightX, rightY, lev, factories)
{
	Cancel( factories );
	lev = Math.abs(lev);
	
	KochSnow(topX, topY, leftX, leftY, lev, factories);
	KochSnow(leftX, leftY, rightX, rightY, lev, factories);	
	KochSnow(rightX, rightY, topX, topY, lev, factories);	

	for ( var i = 0; i < factories.length; ++i )
		factories[i].update();
}

function DoKoch()
{
	var factories = new Array();
	var UI = moi.ui.commandUI;
	Update( UI.topX.numericValue, UI.topY.numericValue, UI.leftX.numericValue, UI.leftY.numericValue, UI.rightX.numericValue, UI.rightY.numericValue, UI.lev.integerValue, factories);

	// Wait for cancel or done.
	var dlg = moi.ui.commandDialog;

	while ( 1 )
	{
		if ( !dlg.waitForEvent() )
		{
			Cancel( factories );
			return false;
		}
			
		if ( dlg.event == 'done' )
			break;
			
		if ( dlg.event == 'topX' || dlg.event == 'topY' || dlg.event == 'leftX' || dlg.event == 'leftY' || dlg.event == 'rightX' || dlg.event == 'rightY' || dlg.event == 'lev' )
		{
			Update( UI.topX.numericValue, UI.topY.numericValue, UI.leftX.numericValue, UI.leftY.numericValue, UI.rightX.numericValue, UI.rightY.numericValue, UI.lev.integerValue, factories);			
		}
	}
	
	Commit( factories );	
}

DoKoch();
