//	MobiusScript produces a Mobius Strip, and is to be run in the MoI program.
//	DRAFT by Brian McMillin July, 2014

// theta, the rotational angle parameter, runs from (0 to 2pi), with ___ held constant.
// psi is used for the lines across the band width of the curve, and varies from (-bandWidth/2 to bandWidth/2).
// For each value of u, one angled line is created around a circle of radius 1.
//	A twist is 180 degrees.
//	For a regular mobius strip, twists = 1.
//  For a plain loop, twists = 2.
//  For a 3-fold or 3-twist mobius strip, twists = 3.
//	The foldPercent is used to localize the twist to a portion, or portions, of the Mobius Strip. 
//	A foldPercent of 100 has the twist uniformly done over the entire length of the Mobius Strip.
//	For a symmetric 3 twist mobius strip, the twist can be done uniformly over the length,
//	or it could be done in three regions

function Cancel( factories )
{
	for ( var i = 0; i < factories.length; ++i )
		factories[i].cancel();
	factories.length = 0;
}

function Commit( factories )
{
	for ( var i = 0; i < factories.length; ++i )
		factories[i].commit();
}

function EvalMobius( tu, tv, bandWidth, twists, xStretch, yStretch )
{
	var theta;
	theta = Math.PI * 2.0 * tu;
//	theta = Math.PI * 0.3 * tu;	
	var uAngle = 0.5 * twists * theta;	
	var psi;	
	psi = bandWidth * 0.5 * ((2 * tv) - 1);
		
	// Define x, y, and z values.
	var x = xStretch * Math.cos(theta) * (1 + psi * Math.cos(uAngle));
	var y = yStretch * Math.sin(theta) * (1 + psi * Math.cos(uAngle));
	var z = psi * ( Math.sin(uAngle)); 	
	return moi.vectorMath.createPoint( x, y, z );
}

/*
function EvalMobius2( tu, tv, bandWidth, twists, xStretch, yStretch )
{
	var theta;
	var psi;

	// Define x, y, and z values.
	var x = Math.cos(theta) * (1 + psi * Math.cos(halfu));
	var y = Math.sin(theta) * (1 + psi * Math.cos(halfu));
	var z = psi * ( Math.sin(halfu)); 	
	return moi.vectorMath.createPoint( x, y, z );
}
//*/

function Update( bandWidth, twists, xStretch, yStretch, uDensity, vDensity, numpoints, factories )
{
	Cancel( factories );
//	twists = Math.abs(twists);

//	NOTE, factory declaration does NOT go here.  It goes 4 lines down.
	for (var j = 0; j < (uDensity - 1); ++j )
	{
	var factory = moi.command.createFactory( 'interpcurve' );
	var tu = j / (uDensity - 1);

		for ( var i = 0; i < numpoints; ++i )
		{
			var tv = i / (numpoints - 1);
		
			var pt = EvalMobius( tu, tv, bandWidth, twists, xStretch, yStretch );		
		
			factory.createInput( 'point' );
			factory.setInput( factory.numInputs - 1, pt );
		}
		factories.push( factory );
//	The next line is a test addition, display the current factory, (one curve).  It works well...
		factory.update();
	}

/*	
//	Now plot the other circles. 	
	for (var j = 0; j < (vDensity - 1); ++j )
	{
	var factory = moi.command.createFactory( 'interpcurve' );

	var tu = j / (vDensity - 1);

		for ( var i = 0; i < numpoints; ++i )
		{
			var tv = i / (numpoints - 1);
		
			var pt = EvalMobius2( tu, tv, bandWidth, twists, xStretch, yStretch );		
		
			factory.createInput( 'point' );
			factory.setInput( factory.numInputs - 1, pt );
		}
		factories.push( factory );
//	The next line is a test addition, display the current factory, (one curve).  It works well...
		factory.update();
	}
//*/
	
		for ( var i = 0; i < factories.length; ++i )
		{
			factories[i].update();
		}	
}

function DoMobius()
{
	var factories = new Array();
	var UI = moi.ui.commandUI
	
	
	Update( UI.bandWidth.numericValue, UI.twists.integerValue, UI.xStretch.numericValue, UI.yStretch.numericValue, UI.uDensity.integerValue, UI.vDensity.integerValue, UI.numpoints.integerValue, factories );

	var dialog = moi.ui.commandDialog;

	while ( 1 )
	{
		if ( !dialog.waitForEvent() )
		{
			Cancel( factories );
			return;
		}
			
		if ( dialog.event == 'done' )
			break;
			
		if ( dialog.event == 'bandWidth' || dialog.event == 'twists' || dialog.event == 'xStretch' || dialog.event == 'yStretch' || dialog.event == 'uDensity' || dialog.event == 'vDensity' || dialog.event == 'numpoints' )
	Update( UI.bandWidth.numericValue, UI.twists.integerValue, UI.xStretch.numericValue, UI.yStretch.numericValue, UI.uDensity.integerValue, UI.vDensity.integerValue, UI.numpoints.integerValue, factories );
	}
			Commit( factories );
}

DoMobius();
