function alert( msg )
{
	moi.ui.commandUI.alert( msg );
}

//  Computes the R_F from Carlson symmetric form
//  http://en.wikipedia.org/wiki/Carlson_symmetric_form#Series_Expansion
    function RF( X, Y, Z )
	{
		//var X, Y, Z;
		var result, A, lamda, dx, dy, dz;
        var MinError = 0.0000001;

        do
		{
            lamda = Math.sqrt(X * Y) + Math.sqrt(Y * Z) + Math.sqrt(Z * X);

            X = (X + lamda) / 4;
            Y = (Y + lamda) / 4;
            Z = (Z + lamda) / 4;

            A = (X + Y + Z) / 3;

            dx = (1 - X / A);
            dy = (1 - Y / A);
            dz = (1 - Z / A);
		}
        while (Math.max(Math.max(Math.abs(dx), Math.abs(dy)), Math.abs(dz)) > MinError);

        var E2, E3;
        E2 = dx * dy + dy * dz + dz * dx;
        E3 = dy * dx * dz;

        //http://dlmf.nist.gov/19.36#E1
        result = 1 - (1 / 10) * E2 + (1 / 14) * E3 + (1 / 24) * E2 * E2 - (3 / 44) * E2 * E3 - (5 / 208) * E2 * E2 * E2  + (3 / 104) * E3 * E3 + (1 / 16) * E2 * E2 * E3;

        result *= (1 / Math.sqrt(A));
        return result;
	}

//  Computes the R_D from Carlson symmetric form
//  http://en.wikipedia.org/wiki/Carlson_symmetric_form#Series_Expansion
//  Constructed from R_J(x,y,z,z) which is equal to R_D(x,y,z)

    function RD( X, Y, Z )
	{
		//var X, Y, Z;
        var sum, fac, lamda, dx, dy, dz, A;
        var MinError = 0.000000001;

        sum = 0;
        fac = 1;

        do
		{
            lamda = Math.sqrt(X * Y) + Math.sqrt(Y * Z) + Math.sqrt(Z * X);
            sum += fac / (Math.sqrt(Z) * (Z + lamda));

            fac /= 4;

            X = (X + lamda) / 4;
            Y = (Y + lamda) / 4;
            Z = (Z + lamda) / 4;

            A = (X + Y + 3 * Z) / 5;

            dx = (1 - X / A);
            dy = (1 - Y / A);
            dz = (1 - Z / A);
		}
        while (Math.max(Math.max(Math.abs(dx), Math.abs(dy)), Math.abs(dz)) > MinError);

        var E2, E3, E4, E5;
        E2 = dx * dy + dy * dz + 3 * dz * dz + 2 * dz * dx + dx * dz + 2 * dy * dz;
        E3 = dz * dz * dz + dx * dz * dz + 3 * dx * dy * dz + 2 * dy * dz * dz + dy * dz * dz + 2 * dx * dz * dz;
        E4 = dy * dz * dz * dz + dx * dz * dz * dz + dx * dy * dz * dz + 2 * dx * dy * dz * dz;
        E5 = dx * dy * dz * dz * dz;

        //http://dlmf.nist.gov/19.36#E2
        var result = (1 - (3 / 14) * E2 + (1 / 6) * E3 + (9 / 88) * E2 * E2 - (3 / 22) * E4 - (9 / 52) * E2 * E3 + (3 / 26) * E5 - (1 / 16) * E2 * E2 * E2 + (3 / 40) * E3 * E3 + (3 / 20) * E2 * E4 + (45 / 272) * E2 * E2 * E3 - (9 / 68) * (E3 * E4 + E2 * E5));

        result = 3.0 * sum + fac * result / (A * Math.sqrt(A));
        return result;
	}


// Returns the incomplete elliptic integral of the first kind using Carlson symmetric forms. 
// u is the angle in radians, from 0 to PI/2 inclusive.
// Note that kk = k*k.   (m = kk)
function ei1F(snValue, kk)
{
	var u = snValue;
	var cosu2 = Math.cos(u);
	cosu2 *= cosu2
	var sinu2 = Math.sin(u);
	sinu2 *= sinu2	
	var result = Math.sin(u) * RF(cosu2, (1 - kk * sinu2), 1)
    return result;
}

// Returns the incomplete elliptic integral of the second kind using Carlson symmetric forms. 
// u is the angle in radians, from 0 to PI/2 inclusive.
// Note that kk = k*k.   (m = kk)
function ei2E(snValue, kk)
{ 
	var u = snValue;
	var result = Math.sin(u) * RF(Math.pow(Math.cos(u),2), 1 - kk * Math.pow(Math.sin(u),2), 1) - (1 / 3) * kk * Math.pow(Math.sin(u),3) * RD(Math.pow(Math.cos(u),2), 1 - kk * Math.pow(Math.sin(u),2), 1)
	return result;
}

function Update(kValue)
{
	var curvefactory = moi.command.createFactory( 'interpcurve' );

	var radiusInit = moi.ui.commandUI.rInit.numericValue;
	var numpoints = moi.ui.commandUI.numpoints.integerValue;
	var kk = kValue*kValue;	
	// ulimit is the complete elliptic integral of the kValue modulus.
	// if kValue = sqrt(2) / 2, then ulimit = 1.8540746773013716 (+/-).
	//	var ulimit = 1.8540746773013716;
	
	var ulimit = ei1F( Math.PI / 2, kk )

	var k = kValue; //var k = Math.sqrt(kValue);	
	for ( var j = 0; j < numpoints; ++j )
	{
		var tval = j / (numpoints - 1);
		var uValue = tval * ulimit;
		//	Get cnValue and snValue.
		var k = kValue;
		var tol = 10.0e-11;
		var snValue;
		var cnValue;
		//	var dn;
		var t;
		/*Set up two arrays and AGM values*/
		var asav = [];
		var csav = [];
		asav[0] = 1.0;
		var b = Math.sqrt(1.0 - kk);
		csav[0] = k; 
		var n2 = 1.0;
		var i = 0;

		while( Math.abs( (csav[i]/asav[i]) ) > tol )
		{
			if( i > 10 )
			{
				/*mtherr( "ellpjf", OVERFLOW );*/
				break;
			}
			var a = asav[i];
			++i;
			csav[i] = 0.5 * ( a - b );
			var t = Math.sqrt( a * b );
			asav[i] = 0.5 * ( a + b );
			b = t;
			n2 += n2;
		}
		/* backward recurrence */
		// Find phi_subzero, starting at a[n] and c[n], and decrementing to n=0.
		// See paragraph 22.20.2 and .3 and .4 of nist website.
		var phi = n2 * asav[i] * uValue;
		do
		{
			t = csav[i] * Math.sin(phi) / asav[i];
			b = phi;
			phi = 0.5 * (Math.asin(t) + phi);
		}
		while( --i );

		snValue = Math.sin(phi);
		t = Math.cos(phi);
		cnValue = t;
		var y = 0.0;
		var x = radiusInit * cnValue;
		var Evalue = ei2E( snValue, kk )
		var Fvalue = ei1F( snValue, kk )
		var z = radiusInit * Math.sqrt(2) * (Evalue - 0.5 * Fvalue);
//		alert ("x =" + x + "\nz =" + z + "\nsnValue =" + snValue + "\ncnValue =" + cnValue + "\nuValue =" + uValue + "\nEvalue =" + Evalue + "\nFvalue =" + Fvalue  );
		//	var pt = cplane.evaluate( x, y, z );
		var pt = moi.vectorMath.createPoint( x, y, z );		
		
		//Add point to the interpcurve factory.
		curvefactory.createInput( 'point' );
		curvefactory.setInput( curvefactory.numInputs - 1, pt );
	}
	curvefactory.update();
	return curvefactory;
}

function DoBalloon()
{
	var kValue = moi.ui.commandUI.kValue.numericValue;

	kValue = Math.abs(kValue);
	
	if (kValue == 0.0)
	{
		kValue = 0.001;
		alert ( " kValue must not = zero, kValue set to 0.001 " ); 
	}	

	if (kValue >= 1.0)
	{
		kValue = 0.9999;
		alert ( " kValue must be less than 1, kValue set to 0.9999 " ); 
	}

// Note that "prefix" moi.ui.commandUI. is needed for code moved from .js to .htm file.
	var curvefactory = Update(kValue);
//	var curvefactory = moi.ui.commandUI.Update(kValue); //for code in .htm file.
	
	var dlg = moi.ui.commandDialog;
	
	while ( 1 )
	{
		if ( !dlg.waitForEvent() )
		{
			curvefactory.cancel();
			return false;
		}
			
		if ( dlg.event == 'done' )
			break;
			
		if ( dlg.event == 'kValue' || dlg.event == 'rInit' || dlg.event == 'numpoints' )
		{
		
			var kValue = moi.ui.commandUI.kValue.numericValue;
			kValue = Math.abs(kValue);
	
			if (kValue == 0.0)
			{
				kValue = 0.001;
				alert ( " kValue must not = zero, kValue set to 0.001 " ); 
			}	

			if (kValue >= 1.0)
			{
				kValue = 0.9999;
				alert ( " kValue must be less than 1, kValue set to 0.9999 " ); 
			}		
		
			curvefactory.cancel();
			var curvefactory = Update(kValue); //for code in .js
			//curvefactory = moi.ui.commandUI.Update(kValue); // for code in .htm
		}
	}
	curvefactory.commit();
}
DoBalloon();







