// By bemfarmer, October 6, 2013
// The Torse (unwrap) of the unit radius Oloid from the parametric
// equations based upon very difficult mathematics of the reference paper.
// Theorem 2 is for the kdA curve.  Theorem 3 is for the kdB curve, which
// also looks like a mirror shifted kdA curve.
// http://www.heldermann-verlag.de/jgg/jgg01_05/jgg0113.pdf

// config: norepeat

#include "GetObjects.js"

function alert( msg )
{
	moi.ui.commandUI.alert( msg );
}

function Cancel( factories )
{
	for ( var i = 0; i < factories.length; ++i )
		factories[i].cancel();
		
	factories.length = 0;
}

function Commit( factories )
{
	for ( var i = 0; i < factories.length; ++i )
		factories[i].commit();
}

function EvalkdAPoint( t )
{
	// From Theorem 2.
	var theta;
	theta = ((Math.PI * 2.0) / 3.0) * t;
	ctheta = Math.cos( theta );	
var x = ((Math.sqrt(3)*2)/9)*(Math.sqrt(2*(1+2*ctheta)*(1-ctheta))+Math.acos((Math.sqrt(2)*ctheta)/Math.sqrt(1+ctheta)));  
	var y = (Math.sqrt(3)/9)*(4*(1-ctheta)+Math.log(2/(1+ctheta)));
	var z = 0.0;
	
	return moi.vectorMath.createPoint( x, y, z );
}

function EvalkdBPoint( t )
{
	var theta;
	theta = ((Math.PI * 2.0) / 3.0) * t;
	ctheta = Math.cos( theta );
var x = ((Math.sqrt(3)*2)/9)*(Math.acos((Math.sqrt(2)*ctheta)/Math.sqrt(1+ctheta))-(Math.sqrt(2*(1-ctheta)*(1+(2*ctheta)))/(1+ctheta)));  
	var y = (Math.sqrt(3)/9)*(Math.log(2/(1+ctheta))+((11+7*ctheta)/(1+ctheta)));
	var z = 0.0;
	
	return moi.vectorMath.createPoint( x, y, z );
}

/*
function EvalA2Point( t )
{
	var theta;
	theta = ((Math.PI * 1.0) / 2.0) * t;
	var x = 0.0;
	var y = 0.5 + Math.cos( theta );
	var z = Math.sin( theta );
	
	return moi.vectorMath.createPoint( x, y, z );
}

function EvalB2Point( t )
{
	var theta;
	theta = ((Math.PI * 1.0) / 2.0) * t;
	cos_theta = Math.cos( theta );
	var x = Math.sqrt(1 + 2*cos_theta) / (1 + cos_theta );
	var y = -0.5 + (cos_theta / (1 + cos_theta ));
	var z = 0.0;
	
	return moi.vectorMath.createPoint( x, y, z );
}
*/

function Update( numpoints, factories )
{
	Cancel( factories );
	
	// Build kdA torse curve.	
	var factoryA = moi.command.createFactory( 'interpcurve' );
	// t is a fraction, and 0 <= t <= 1	
	for ( var i = 0; i < (numpoints); ++i )
	{
		var t = i / (numpoints-1);
		var Apt = EvalkdAPoint( t );

		factoryA.createInput( 'point' );
		factoryA.setInput( factoryA.numInputs - 1, Apt );
	}
	factories.push( factoryA );
	
	// Build kdB torse curve.
	var factoryB = moi.command.createFactory( 'interpcurve' );
	for ( var i = 0; i < (numpoints); ++i )
	{
		var t = i / (numpoints-1);
		var Bpt = EvalkdBPoint( t );
		factoryB.createInput( 'point' );
		factoryB.setInput( factoryB.numInputs - 1, Bpt );
	}
	factories.push( factoryB );
	
	for ( var i = 0; i < factories.length; ++i )
		factories[i].update();
}

function DoOloidTorse()
{
	var factories = new Array();
	Update( moi.ui.commandUI.numpoints.integerValue, factories );
	
	// Wait for cancel or done.	
	var dlg = moi.ui.commandDialog;
	while ( 1 )
	{
		if ( !dlg.waitForEvent() )
		{
			Cancel( factories );
			return;
		}
			
		if ( dlg.event == 'done' )
			break;
			
		if ( dlg.event == 'numpoints' )
			Update( moi.ui.commandUI.numpoints.integerValue, factories );
	}
	
	Commit( factories );
}

DoOloidTorse();
