//	Plücker's conoid by Brian McMillin June 18, 2014.
//	This script for Moment of Inspiration, generates a ruled surface known as Pluckers Conoid.
//  Plücker's conoid is associated with hyperboloid gears, and has been used for roofs.

//	v = radius
//	u = theta, and ranges from 0 to 2PI.
//  negative zHeight or negative fold number "rotates" or "mirrors" conoid. 
//  "CenterCurveSkips" removes curves starting from the center, and reduces curve count.
//  Color assignment could likely be simpler.

var g_style_index = 1;

function AssignColors( factory )
{
	var styles = moi.geometryDatabase.getObjectStyles();
	var curves = factory.getCreatedObjects();
//	for ( var i = 0; i < curves.length; ++i, ++g_style_index )
	for ( var i = 0; i < curves.length; ++i )	
	{
		if ( g_style_index == styles.length )
		{
			g_style_index = 0;
		}
		var curve = curves.item(i);
		curve.styleIndex = g_style_index;
	}
}

function Cancel( factories )
{
	for ( var i = 0; i < factories.length; ++i )
		factories[i].cancel();
	factories.length = 0;
}

function Commit( factories )
{
	for ( var i = 0; i < factories.length; ++i )
		factories[i].commit();
}

function EvalPluckerConoid( tu, zHeight, foldValue, rad )
{
	var theta;
	theta = Math.PI * 2.0 * tu;
	// Define x, y, and z values.
	var x = rad * Math.cos(theta);
	var y = rad * Math.sin(theta);
	var z = zHeight * Math.sin( foldValue * theta); 	
	return moi.vectorMath.createPoint( x, y, z );
}

function Update( zHeight, foldValue, maxRad, skips, uDensity, vDensity, numpoints, factories, lfactories )
{
	Cancel( factories );
	Cancel( lfactories );	
	skips = Math.abs(skips);
	uDensity = Math.abs(uDensity);
	vDensity = Math.abs(vDensity);
	numpoints = Math.abs(numpoints);
//	NOTE, factory declaration does NOT go here.  It goes in the u for loop.
//	For each radius v, plot a curve where rotational angle parameter theta = u, ranges from 0 to 2PI.
	for (var j = skips; j < (vDensity); ++j )  //not vDensity-1	
	{
	var factory = moi.command.createFactory( 'interpcurve' );
//	var factory = moi.command.createFactory( 'curve' );
	var tv = j / (vDensity - 1);
	var rad = maxRad * tv;
	  {
		for ( var i = 0; i < numpoints; ++i )
		{
			var tu = i / (numpoints - 1);
		
			var pt = EvalPluckerConoid( tu, zHeight, foldValue, rad );		
		
			factory.createInput( 'point' );
			factory.setInput( factory.numInputs - 1, pt );
		}
	  }
		factories.push( factory );
		// The next codeline displays curves as they are created.
		factory.update();
	}

//	For each angle u, plot a line, where radius parameter v ranges from minRad to maxRad.
	for (var j = 0; j < (uDensity - 1); ++j )
	{
	var linefactory = moi.command.createFactory( 'line' );
	var tu = j / (uDensity - 1);
		// Each line requires only two points.
		var tv_min = skips / (vDensity - 1);
		var rad = maxRad * tv_min;			
		var pt = EvalPluckerConoid( tu, zHeight, foldValue, rad );		
		linefactory.setInput( 0, pt );
		var rad = maxRad;
		var pt = EvalPluckerConoid( tu, zHeight, foldValue, rad );		
		linefactory.setInput( 1, pt );
		lfactories.push( linefactory );
		// The next codeline displays lines as they are created.
		linefactory.update();
	}

	for ( var i = 0; i < factories.length; ++i )
		{
			factories[i].update();
		}
		

		for ( var i = 0; i < lfactories.length; ++i )
		{
			lfactories[i].update();
			AssignColors( lfactories[i] );			
		}
}

function DoPluckerConoid()
{
	var factories = new Array();
	var lfactories = new Array();
	var UI = moi.ui.commandUI; //Abbreviation
	
	Update( UI.zHeight.numericValue, UI.foldValue.integerValue, UI.maxRad.numericValue, UI.skips.integerValue, UI.uDensity.integerValue, UI.vDensity.integerValue, UI.numpoints.integerValue, factories, lfactories );

	var dialog = moi.ui.commandDialog;

	while ( 1 )
	{
		if ( !dialog.waitForEvent() )
		{
			Cancel( factories );
			return;
		}
			
		if ( dialog.event == 'done' )
			break;
			
		if ( dialog.event == 'zHeight' || dialog.event == 'foldValue' || dialog.event == 'maxRad' || dialog.event == 'skips' || dialog.event == 'uDensity' || dialog.event == 'vDensity' || dialog.event == 'numpoints' )
		{
g_style_index = 1;//reset color start index
Update( UI.zHeight.numericValue, UI.foldValue.integerValue, UI.maxRad.numericValue, UI.skips.integerValue, UI.uDensity.integerValue, UI.vDensity.integerValue, UI.numpoints.integerValue, factories, lfactories );
		}
	}
			Commit( factories );
			Commit( lfactories );
}

DoPluckerConoid();
