//	Scherk's First Minimal Surface by Brian McMillin February 20, 2014
//   
//	phi = u, and tau =v, are angle parameters whose interval is (-PI/2, PI/2), inclusive. 
//	A larger interval does not work.  This is probably due to division by zero?
//  +/- PI/2 works, even though the cosine of PI/2 is (very nearly?) zero.
//	A small absolute value for cValue makes the "saddle" flatter.
//	A negative cValue faces the horse in the direction of the x axis.
//  The maximum absolute value for cValue is coded to 1.0.
//	Network works for cValue = .99.	

function Cancel( factories )
{
	for ( var i = 0; i < factories.length; ++i )
		factories[i].cancel();
		
	factories.length = 0;
}

function Commit( factories )
{
	for ( var i = 0; i < factories.length; ++i )
		factories[i].commit();
}

function EvalScherk1( c, tu, tv )
{
	var phi = Math.PI * (tu - 0.5);
	var tau = Math.PI * (tv - 0.5);	
	
	var x = phi;
	var y = tau;
	var z = Math.log(Math.cos(c*phi)/Math.cos(c*tau))/c;
 
	return moi.vectorMath.createPoint( x, y, z );
}

function Update( cValue, unum, vnum, ptsnum, factories )
{
	Cancel( factories );
	if (Math.abs(cValue) > 1.0)
		cValue = 1.0;
	if (Math.abs(cValue) == 0)
		cValue = 1.0;
	//	Plot the v curves, if box is checked.
	if ( moi.ui.commandUI.vCurvesCheckbox.value )	
	{
		for (var j = 0; j < (unum); ++j )		
		{
			var factory = moi.command.createFactory( 'interpcurve' );
			var tu = j / (unum-1);
			for ( var i = 0; i < (ptsnum); ++i )		
			{
				var tv = i / (ptsnum - 1);
				var pt = EvalScherk1( cValue, tu, tv );		
		
				factory.createInput( 'point' );
				factory.setInput( factory.numInputs - 1, pt );
			}

		factories.push( factory );
		//  Display the current factory.
		factory.update();
	}
	}
	
	// Plot the u curves, if box is checked.
	if ( moi.ui.commandUI.uCurvesCheckbox.value )
	{	
		for (var j = 0; j < (vnum); ++j )		
		{
			var factory = moi.command.createFactory( 'interpcurve' );
			var tv = j / (vnum - 1);
			for ( var i = 0; i < ptsnum; ++i )
			{
				var tu = i / (ptsnum - 1);
				var pt = EvalScherk1( cValue, tu, tv );		
		
				factory.createInput( 'point' );
				factory.setInput( factory.numInputs - 1, pt );
			}

			factories.push( factory );
			//	Display the current factory.
			factory.update();
		}	
	}
		for ( var i = 0; i < factories.length; ++i )
		{
			factories[i].update();
		}	
}

function DoScherk1()
{
	var factories = new Array();
	var ui = moi.ui.commandUI
	
	Update( ui.cValue.numericValue, ui.unum.integerValue, ui.vnum.integerValue, moi.ui.commandUI.ptsnum.integerValue, factories );

	var dialog = moi.ui.commandDialog;

	while ( 1 )
	{
		if ( !dialog.waitForEvent() )
		{
			Cancel( factories );
			return;
		}
			
		if ( dialog.event == 'done' )
			break;
			
		if ( dialog.event == 'cValue' || dialog.event == 'unum' || dialog.event == 'vnum' || dialog.event == 'ptsnum' || dialog.event == 'uCurvesCheckbox' || dialog.event == 'vCurvesCheckbox' )
			Update( ui.cValue.numericValue, ui.unum.integerValue, ui.vnum.integerValue, moi.ui.commandUI.ptsnum.integerValue, factories );
	}
			Commit( factories );
}

DoScherk1();
