//	Scherk's second minimal surface by Brian McMillin February 17, 2014
//   
//	r = u, is the radius parameter whose interval is (0 to 1). (Includes 0 and 1). (Or slightly more than 1 can be used)
//	phi = v, is the rotational angle parameter whose interval is (0, 2pi], pi excluded.  It is supposed to be [0, 2pi), 0 excluded.

function Cancel( factories )
{
	for ( var i = 0; i < factories.length; ++i )
		factories[i].cancel();
		
	factories.length = 0;
}

function Commit( factories )
{
	for ( var i = 0; i < factories.length; ++i )
		factories[i].commit();
}

function EvalScherk2( tu, tv )
{
	var phi = Math.PI * 2.0 * tv;
	var r = tu;
	rsq = r*r;
	cosphi = 2*r*Math.cos(phi);
	sinphi = 2*r*Math.sin(phi);
	// Define x, y, and z values.
	var x = Math.log((1 + rsq + cosphi) / (1 + rsq - cosphi));
	var y = Math.log((1 + rsq - sinphi) / (1 + rsq + sinphi));
	var z = 2*Math.atan((2*rsq*Math.sin(2*phi))/(rsq*rsq - 1));
// 	var z = 2*Math.atan2((2*rsq*Math.sin(2*phi)),(rsq*rsq -1));
 
	return moi.vectorMath.createPoint( x, y, z );
}

//function Update( unum, vnum, ptsnum, factories )
function Update( unum, vnum, ptsnum, factories )
{
	Cancel( factories );
	
	//	Plot the "kind of concentric" curves.
	if ( moi.ui.commandUI.ConcentricCurvesCheckbox.value )	
	{
//		for (var j = 0; j < (unum); ++j )

		for (var j = 0; j < (unum); ++j )		
		{
			var factory = moi.command.createFactory( 'interpcurve' );
			// Avoid division by zero, r==1 is forbidden;			
			var tu = j / (unum);
//			if (tu == 1)
//				tu = 0.99999;
			for ( var i = 0; i < (ptsnum); ++i )		
			{
				var tv = i / (ptsnum - 1);
				if (tv == 0)
					tv = 1.0; // Use 2PI for angle phi, instead of forbidden zero angle, so "concentric" curves close.
				var pt = EvalScherk2( tu, tv );		
		
				factory.createInput( 'point' );
				factory.setInput( factory.numInputs - 1, pt );
			}

		factories.push( factory );
		//  Display the current factory.
		factory.update();
	}
	}
	
	// Build "kind of radial" curves if box checked.
	// Angle phi, or tv, cannot be zero.
	if ( moi.ui.commandUI.RadialCurvesCheckbox.value )
	{	
		for (var j = 1; j < (vnum); ++j )		
		{
			var factory = moi.command.createFactory( 'interpcurve' );
			var tv = j / (vnum - 1);
				
			for ( var i = 0; i < ptsnum; ++i )
			{
				var tu = i / (ptsnum - 1);
				if (tu == 1)
					tu = 0.99999;
				var pt = EvalScherk2( tu, tv );		
		
				factory.createInput( 'point' );
				factory.setInput( factory.numInputs - 1, pt );
			}

			factories.push( factory );
			//	Display the current factory.
			factory.update();
		}	
	}
		for ( var i = 0; i < factories.length; ++i )
		{
			factories[i].update();
		}	
}

function DoScherk2()
{
	var factories = new Array();
	var ui = moi.ui.commandUI
	
	Update( ui.unum.integerValue, ui.vnum.integerValue, moi.ui.commandUI.ptsnum.integerValue, factories );

	var dialog = moi.ui.commandDialog;

	while ( 1 )
	{
		if ( !dialog.waitForEvent() )
		{
			Cancel( factories );
			return;
		}
			
		if ( dialog.event == 'done' )
			break;
			
		if ( dialog.event == 'unum' || dialog.event == 'vnum' || dialog.event == 'ptsnum' || dialog.event == 'RadialCurvesCheckbox' || dialog.event == 'ConcentricCurvesCheckbox' )
			Update( ui.unum.integerValue, ui.vnum.integerValue, moi.ui.commandUI.ptsnum.integerValue, factories );
	}
			Commit( factories );
}

DoScherk2();
