/*
SphereHarmony is a script to create bumpy or distorted spheres, using spherical coordinates, (radius, theta, phi).
The formula is from the Paul Bourke website, and is NOT the formula for Mathematical spherical harmonics.  Atomic orbitals are not produced.
http://paulbourke.net/geometry/sphericalh/

Network and loft tend to work less well for higher values of m4 through m7.  

Moi javascript version by Brian McMillin, August 1-14, 2015.
*/

function Cancel( factories )
{
	for ( var i = 0; i < factories.length; ++i )
		factories[i].cancel();
		
	factories.length = 0;
}

function Commit( factories )
{
	for ( var i = 0; i < factories.length; ++i )
		factories[i].commit();
}

function EvalSH( tu, t2, m0, m1, m2, m3, m4, m5, m6, m7 )
{
	var theta;
	var phi;
	var r = 0.0;
	phi =  Math.PI * tu;
	theta = Math.PI * 2.0 * t2;
	r += Math.pow(Math.sin( m0*phi ), m1 );
	r += Math.pow(Math.cos( m2*phi ), m3 );
	r += Math.pow(Math.sin( m4*theta ), m5 );
	r += Math.pow(Math.cos( m6*theta ), m7 );
	var sinphi = Math.sin(phi);
	// Define x, y, and z values.
	var x = r * sinphi * Math.cos(theta);
	var y = r * Math.cos(phi);
	var z = r * sinphi * Math.sin(theta); 	
	return moi.vectorMath.createPoint( x, y, z );
}

function Update( m0, m1, m2, m3, m4, m5, m6, m7, uDensity, vDensity, numpoints, factories )
{
	Cancel( factories );
//	for (var j = 0; j < (uDensity - 1); ++j )
	for (var j = 1; j < (uDensity - 1); ++j )		
	{
	var factory = moi.command.createFactory( 'interpcurve' );
//	var factory = moi.command.createFactory( 'curve' );
	var tu = j / (uDensity - 1);

		for ( var i = 0; i < numpoints; ++i )
		{
			var t2 = i / (numpoints - 1);
		
			var pt = EvalSH( tu, t2, m0, m1, m2, m3, m4, m5, m6, m7 );		
		
			factory.createInput( 'point' );
			factory.setInput( factory.numInputs - 1, pt );
		}

		factories.push( factory );
		factory.update();
	}
	
//	Now plot the other curves. 	
	for (var j = 0; j < (vDensity - 1); ++j )
	{
	var factory = moi.command.createFactory( 'interpcurve' );
//	var factory = moi.command.createFactory( 'curve' );
	var tv = j / (vDensity - 1);

		for ( var i = 0; i < numpoints; ++i )
		{
			var t2 = i / (numpoints - 1);
		
			var pt = EvalSH( t2, tv, m0, m1, m2, m3, m4, m5, m6, m7 );		
		
			factory.createInput( 'point' );
			factory.setInput( factory.numInputs - 1, pt );
		}

		factories.push( factory );
		factory.update();
	}	
	
		for ( var i = 0; i < factories.length; ++i )
		{
			factories[i].update();
		}	
}

function DoSphHarmony()
{
	var factories = new Array();
	var UI = moi.ui.commandUI
	
	Update( UI.m0.integerValue, UI.m1.integerValue, UI.m2.integerValue, UI.m3.integerValue, UI.m4.integerValue, UI.m5.integerValue, UI.m6.integerValue, UI.m7.integerValue, UI.uDensity.integerValue, UI.vDensity.integerValue, UI.numpoints.integerValue, factories );

	var dialog = moi.ui.commandDialog;

	while ( 1 )
	{
		if ( !dialog.waitForEvent() )
		{
			Cancel( factories );
			return;
		}
			
		if ( dialog.event == 'done' )
			break;
			
		if ( dialog.event == 'm0' || dialog.event == 'm1' || dialog.event == 'm2' || dialog.event == 'm3' || dialog.event == 'm4' || dialog.event == 'm5' || dialog.event == 'm6' || dialog.event == 'm7' || dialog.event == 'uDensity' || dialog.event == 'vDensity' || dialog.event == 'numpoints' )
			Update( UI.m0.integerValue, UI.m1.integerValue, UI.m2.integerValue, UI.m3.integerValue, UI.m4.integerValue, UI.m5.integerValue, UI.m6.integerValue, UI.m7.integerValue, UI.uDensity.integerValue, UI.vDensity.integerValue, UI.numpoints.integerValue, factories );
	}
			Commit( factories );
}

DoSphHarmony();
