/*
This simple script models a teardrop shape.  
http://paulbourke.net/geometry/teardrop/

Moi javascript version by Brian McMillin, August 12, 2015.

*/

function Cancel( factories )
{
	for ( var i = 0; i < factories.length; ++i )
		factories[i].cancel();
		
	factories.length = 0;
}

function Commit( factories )
{
	for ( var i = 0; i < factories.length; ++i )
		factories[i].commit();
}

function EvalT( tu, t2, s1, s2, height )
{
	var theta;
	var phi;

	theta =  Math.PI * tu;
	phi =  Math.PI * 2.0 * t2;
	
	// Define x, y, and z values.
	var x = s1 * (1 - Math.cos(theta)) * Math.sin(theta) * Math.cos(phi);	
	
	var y = s2 * (1 - Math.cos(theta)) * Math.sin(theta) * Math.sin(phi);	
	
	var z = height * Math.cos(theta); 	

	return moi.vectorMath.createPoint( x, y, z );
}

function Update( s1, s2, height, uDensity, vDensity, numpoints, factories )
{
	Cancel( factories );
	
//  Plot "latitude curves."	
	for (var i = 0; i < (uDensity-1); ++i )
	{
	var factory = moi.command.createFactory( 'interpcurve' );
//	var factory = moi.command.createFactory( 'curve' );
	var tu = i / (uDensity - 1);

	for ( var j = 0; j < (numpoints); ++j )
		{
			var t2 = j / (numpoints - 1);
			var pt = EvalT( tu, t2, s1, s2, height );		
			factory.createInput( 'point' );
			factory.setInput( factory.numInputs - 1, pt );
		}
		factories.push( factory );
		factory.update();
	}

//	Plot "longitude curves."	
	for (var i = 0; i < (vDensity-1); ++i )
	{
	var factory = moi.command.createFactory( 'interpcurve' );
//	var factory = moi.command.createFactory( 'curve' );
	var tv = i / (vDensity - 1);

		for ( var j = 0; j < (numpoints); ++j )
		{
			var t2 = j / (numpoints - 1);
			var pt = EvalT( t2, tv, s1, s2, height );		
			factory.createInput( 'point' );
			factory.setInput( factory.numInputs - 1, pt );
		}

		factories.push( factory );
		factory.update();
	}
	
		for ( var i = 0; i < factories.length; ++i )
		{
			factories[i].update();
		}	
}

function DoTear()
{
	var factories = new Array();
	var UI = moi.ui.commandUI
	
	Update( UI.s1.numericValue, UI.s2.numericValue, UI.height.numericValue, UI.uDensity.integerValue, UI.vDensity.integerValue, UI.numpoints.integerValue, factories );

	var dialog = moi.ui.commandDialog;

	while ( 1 )
	{
		if ( !dialog.waitForEvent() )
		{
			Cancel( factories );
			return;
		}
			
		if ( dialog.event == 'done' )
			break;
			
		if ( dialog.event == 's1' || dialog.event == 's2' || dialog.event == 'height' || dialog.event == 'uDensity' || dialog.event == 'vDensity' || dialog.event == 'numpoints' )
			Update( UI.s1.numericValue, UI.s2.numericValue, UI.height.numericValue, UI.uDensity.integerValue, UI.vDensity.integerValue, UI.numpoints.integerValue, factories );
	}
			Commit( factories );
}

DoTear();
