// Variable Pitch Helix
// By Brian McMillin, May 14, 2014
// Note: For non-variable-radius helix, use the regular MoI Helix command.
// Based upon mathematics here:
// http://www.zwsoft.com/forum/thread-2980-1-1.html
// Note that there are several spelling errors.
// The attached VariablePitchHelix pdf shows the simple algebra in more detail.
// sP is the starting pitch of the first 360 degree curve. The user enters this length.  
// eP is the end pitch of the last  360 degree curve. The user enters this length.
// N = numTurns, and is calculated from L, sP and eP.
// p1 is the parameter related to fixed pitch.
// p2 is the parameter related to variable pitch.
// sP, eP, p1, and p2 should not to be confused with pitches which could be calculated from tangents at the first and last points of the helix.

function alert( msg )
{
	moi.ui.commandUI.alert( msg );
}

function EvalVPHelix( t, radius1, radius2, p1, p2, N, reverse )
{
	var R = radius1 + radius2 * t;
	var theta;
	theta = Math.PI * 2.0 * N * t;
	var xy1 = R * Math.sin( theta );
	var xy2 = R * Math.cos( theta );
	
	if (reverse)
	{
		var x = xy2;
		var y = xy1;
	}
	else
	{
		var x = xy1;
		var y = xy2;
	}
	var z = N * (p1 * t + p2 * t * t);

	return moi.vectorMath.createPoint( x, y, z );
}

function Update()
{
	var UI = moi.ui.commandUI;
	var factory = moi.command.createFactory( 'interpcurve' );

	var Length = UI.Length.numericValue;
	var radius1 = UI.radius1.numericValue;
	var radius2 = UI.endRadius.numericValue - radius1;
	var sP = UI.sP.numericValue;
	var eP = UI.eP.numericValue;
	var reverse = UI.ReverseTwist.value;	
	var sum = sP + eP;
	// Division by 0 would be a problem.
	// This should probably be catch(e) coded???
	if (sum == 0.0)
	{
//		sum = 10.0;
	eP += 10;
		alert ( "sP + eP = 0, try again" );
	}
	
	var N = 2 * Length / (sP + eP);
	if (N == 0.0 || N == 1.0)
	{
		N += 0.1;
		alert ( "N = 0, or N-1 = 0, Check Length" );
	}	
	var numpoints = UI.numpts.integerValue * N;

	var p2 = ( N/(N-1)) * (eP - sP)/2;
	var p1 = Length/N - p2;
	
	
//	alert ( " N = " + N +  "   p2 = " + p2 + "   p1 = " + p1 );
	
	for ( var i = 0; i < numpoints; ++i )
	{
		var t = i / (numpoints - 1);
		
		var pt = EvalVPHelix( t, radius1, radius2, p1, p2, N, reverse );	
		
		factory.createInput( 'point' );
		factory.setInput( factory.numInputs - 1, pt );
	}
	
	factory.update();
	
	return factory;
}

function DoVPHelix()
{
	var factory = Update();

	var ui = moi.ui;	
	var dlg = moi.ui.commandDialog;
	
	while ( 1 )
	{
		if ( !dlg.waitForEvent() )
		{
			factory.cancel();
			return false;
		}
			
		if ( dlg.event == 'done' )
			break;
			
		if ( dlg.event == 'Length' || dlg.event == 'radius1' || dlg.event == 'endRadius' || dlg.event == 'sP'|| dlg.event == 'eP' || dlg.event == 'numpts' || dlg.event == 'ReverseTwist' )
		{
			factory.cancel();
			factory = Update();
		}
	}
	
	factory.commit();
}

DoVPHelix();
