//config: norepeat

//Fork-version  2013.11.20

#include "WaitForDialogDone.js"

function makeHexRod( ratio ) {
	
	moi.ui.beginUIUpdate();
	moi.ui.showUI( 'TitleContainer' );
	moi.ui.hideUI( 'InputContainer' );
	moi.ui.hideUI( 'done' );
	moi.ui.hideUI( 'cancel' );
	moi.ui.endUIUpdate();

	//Get values from the UI input fields	
	var wrenchSize = moi.ui.commandUI['wrenchSize'].numericValue / ratio;
	var rodHeight = moi.ui.commandUI['rodHeight'].numericValue / ratio;
	var chamferAngle = moi.ui.commandUI['chamferAngle'].value;
	var chamferTop = moi.ui.commandUI['chamferTop'].value;
	var chamferBottom = moi.ui.commandUI['chamferBottom'].value;
	var chamferDepth = moi.ui.commandUI['chamferDepth'].value/100+1;
	var innerRadius = wrenchSize / 2;
	var outerRadius =  wrenchSize / Math.sqrt(3);
	if ( wrenchSize*rodHeight == 0 ) { return; }
	//hide and protect what is already here
	moi.geometryDatabase.selectAll();
	moi.geometryDatabase.hide();
		
	//Make round blank
	factory = moi.command.createFactory( 'cylinder' );
	factory.setInput( 1, moi.vectorMath.createTopFrame() );
	factory.setInput( 3, outerRadius ); //Radius
	factory.setInput( 4, moi.vectorMath.createPoint( 0, 0, rodHeight ) ); //Height
	factory.update();
	var blank = factory.getCreatedObjects();
	factory.commit();
	
	if ( chamferAngle > 0 && chamferAngle <= 60 ) {
		//Chamfer end(s)
		var list = moi.geometryDatabase.createObjectList();	
		if ( chamferTop ) list.addObject( blank.item(0).getEdges().item(2) ); //Chamfer top end
		if ( chamferBottom ) list.addObject( blank.item(0).getEdges().item(0) ); //Chamfer bottom end 
		factory = moi.command.createFactory( 'chamfer' );
		factory.setInput( 0, list );
		factory.setInput( 3, (outerRadius - innerRadius)*chamferDepth);
		factory.setInput( 4, ( outerRadius - innerRadius )*chamferDepth* Math.tan( chamferAngle * Math.PI / 180 ) ); 
		factory.commit();
		moi.geometryDatabase.selectAll();
		blank = moi.geometryDatabase.getSelectedObjects(); //update
	}
	
	//Draw hex shape
	factory = moi.command.createFactory( 'polygon' );
	factory.setInput( 0, moi.vectorMath.createTopFrame() ); //Center
	factory.setInput( 1, moi.vectorMath.createPoint( innerRadius, 0, 0 ) ); //Radius
	factory.setInput( 2, 6 ); //Num sides
	factory.setInput( 3, true ); //circumscribed?
	factory.update();
	var hexShape = factory.getCreatedObjects(); 
	factory.commit();

	//Cut rod into hex shape
	factory = moi.command.createFactory( 'booleanintersection' );
	factory.setInput( 0, blank );
	factory.setInput( 1, hexShape );
	factory.commit();
	
	moi.geometryDatabase.removeObject( hexShape.item(0) );
	
	//Unhide what was already here before	
	moi.geometryDatabase.deselectAll();
	moi.geometryDatabase.hide(); 
	moi.geometryDatabase.deselectAll();
}


if ( WaitForDialogDone() ) 
{
	switch (moi.geometryDatabase.Units)
	{
		case 'Millimeters': makeHexRod(1);
			break;
		case 'Centimeters': makeHexRod(10);
			break;
		case 'Meters': makeHexRod(1000);
			break;
		case 'Inches': makeHexRod(25.4);
			break;
		case 'Feet': makeHexRod(304.8);
			break;
		default:  moi.ui.alert( "Incompatible unit system!\n\nPlease use the Options dialog to switch the Unit system to\nMillimeters, Centimeters, Meters, Inches or Feet and run the script again.", title="Wrong Unit System" );
	}
}