// config: norepeat

#include "WaitForDialogDone.js"

function round( x, n )
{
	var a = Math.pow( 10, n );
	return Math.round( x * a ) / a;
}

function makeThreadTap() {
		
	//Get values from the UI input fields	
	var threadDiameter = moi.ui.commandUI['threadSize'].value.split("x")[0] * 1.0; //use part before "x" as diameter
	var threadPitch = moi.ui.commandUI['threadSize'].value.split("x")[1] * 1.0; //use part after "x" as pitch
	var threadLength = moi.ui.commandUI['threadLength'].numericValue * 1.0;
	var reverse = moi.ui.commandUI['reverse'].value;
	
	//http://de.wikipedia.org/wiki/Metrisches_ISO-Gewinde
	var t = threadPitch * Math.cos(30 * Math.PI / 180);
	var helixRadius = threadDiameter / 2 + t / 8 - t;
	var rodRadius = helixRadius + t / 4;
	var tipRadius = threadPitch * t / 12;
	
	moi.ui.beginUIUpdate();
	moi.ui.showUI( 'TitleContainer' );
	moi.ui.hideUI( 'InputContainer' );
	moi.ui.hideUI( 'done' );
	moi.ui.hideUI( 'cancel' );
	moi.ui.endUIUpdate();
	
	//hide and protect what is already here
	moi.geometryDatabase.selectAll();
	moi.geometryDatabase.hide(); 
	
	//Draw thread cutting profile
	frame = moi.vectorMath.createFrontFrame();
	frame.origin = moi.vectorMath.createPoint( helixRadius, 0, threadPitch / 2 );
	factory = moi.command.createFactory( 'polygonedge' );
	factory.setInput( 0, frame );
	factory.setInput( 1, moi.vectorMath.createPoint( helixRadius, 0, -1 * threadPitch / 2 ) );
	factory.setInput( 2, 3 );
	factory.commit();
	moi.geometryDatabase.selectAll();
	var profile = moi.geometryDatabase.getSelectedObjects();

	//Round off the tip of the thread cutting profile
	factory = moi.command.createFactory( 'fillet' );
	var vertexflags = moi.createList();
	vertexflags.add( false );
	vertexflags.add( false );
	vertexflags.add( true ); //select the third vertex (corner point)
	factory.setInput( 2, vertexflags );
	factory.setInput( 0, moi.geometryDatabase.getSelectedObjects() );
	factory.setInput( 3, tipRadius );
	factory.setInput( 4, "circular" );
	factory.commit();
	moi.geometryDatabase.selectAll();
	profile = moi.geometryDatabase.getSelectedObjects(); //update profile
	moi.geometryDatabase.getSelectedObjects().item(0).name = 'profile';
	
	//Save profile for later
	moi.geometryDatabase.copyToClipboard(profile); 
	
	//Make thread helix
	moi.geometryDatabase.selectAll();
	var frame = moi.vectorMath.createTopFrame();
	frame.origin = moi.vectorMath.createPoint( 0, 0, 0 );
	var factory = moi.command.createFactory( 'helix' );
	factory.setInput( 0, frame ); //Start point
	factory.setInput( 1, moi.vectorMath.createPoint( 0, 0, threadLength ) ); //End point
	factory.setInput( 2, moi.vectorMath.createPoint( helixRadius, 0, 0 ) ); //Radius point
	factory.setInput( 7, threadPitch ); //Pitch
	factory.setInput( 8, 'flat' );
	factory.setInput( 9, reverse ); 
	factory.commit();
	
	//Reset viewports
	moi.geometryDatabase.invertSelection();
	var rail = moi.geometryDatabase.getSelectedObjects();
	moi.view.resetAll();
	
	//Make thread sweep
	moi.geometryDatabase.selectAll();
	var list = moi.createList();
	list.add( moi.vectorMath.createPoint( 0, 0, 0 ) );
	list.add( moi.vectorMath.createPoint( 0, 0, 1 ) );
	factory = moi.command.createFactory( 'sweep' );
	factory.setInput( 0, profile );
	factory.setInput( 1, rail );
	factory.setInput( 2, list );
	factory.setInput( 4, 'Regular' ); //Ends
	factory.setInput( 5, 'Flat' ); //Twist
	factory.setInput( 7, true ); //Cap ends
	factory.setInput( 10, 'Exact' ); //Profiles
	factory.commit();
	
	//Delete profile + rail
	factory = moi.command.createFactory( 'delete' );
	factory.setInput( 0, moi.geometryDatabase.getSelectedObjects() );
	factory.commit();
		
	//Make rod
	moi.geometryDatabase.selectAll();
	frame = moi.vectorMath.createTopFrame();
	frame.origin = moi.vectorMath.createPoint( 0, 0, -1 * threadLength);
	factory = moi.command.createFactory( 'cylinder' );
	factory.setInput( 1, frame );
	factory.setInput( 3, rodRadius );
	factory.setInput( 4, moi.vectorMath.createPoint( 0, 0, threadLength + threadPitch ) );
	factory.commit();
	moi.geometryDatabase.invertSelection();
	var cylinder = moi.geometryDatabase.getSelectedObjects().item(0);
	moi.geometryDatabase.getSelectedObjects().item(0).name = "cylinder";
	
	//Combine rod and thread sweep
	moi.geometryDatabase.selectAll();
	factory = moi.command.createFactory( 'booleanunion' );
	factory.setInput( 0, moi.geometryDatabase.getSelectedObjects() );
	factory.commit();
		
	//Reset viewports
	moi.geometryDatabase.selectAll();
	moi.view.resetAll();
	
	//Unhide what was already here before	
	moi.geometryDatabase.deselectAll();
	moi.geometryDatabase.hide(); 
	moi.geometryDatabase.deselectAll();

} 

if (moi.geometryDatabase.Units == "Millimeters") {

	if ( WaitForDialogDone() ) makeThreadTap();
	
} else {
	
	moi.ui.alert( "This scripts only accepts input in Millimeters!\n\nPlease use the Options dialog to switch the Unit system to Millimeters\nand run the script again.", title="Wrong Unit System" );
}